/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.2
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type LinuxKernelDirect struct {
	KernelFilePath string `json:"KernelFilePath,omitempty"`

	InitRdPath string `json:"InitRdPath,omitempty"`

	KernelCmdLine string `json:"KernelCmdLine,omitempty"`
}
